/*
 * phase.c
 */

#include "model.h"

extern Firm firm[NFirm];
extern Worker worker[NWorker];
extern Consumer consumer[NConsumer];
extern Holder holder[NHolder];
extern Env env;
extern long seed;

void preF(){
	int i;

	for(i=0;i<NFirm;i++){
		firm[i].demand = 0.0;
		firm[i].invest = 0.0;
	}
}

void investF(){
	int h,i,choice,choiceNum,t;

	for(h=0;h<NHolder;h++){
		choiceNum = (int)(ran1(&seed) * Memory);
		choice = holder[h].list.firm[choiceNum];
		holder[h].choice = choice;
		(firm[choice].invest) += (env.iunit);
	}
	for(i=0;i<NFirm;i++)
		firm[i].stock = (1.0 - env.depr) * firm[i].stock + firm[i].invest;
}

void jobF(){
	int h,i,choice,choiceNum;
	double aveW;

	for(h=0;h<NWorker;h++){
		aveW = 0.0;
		for(i=0;i<Memory;i++) aveW += firm[(worker[h].list.firm[i])].wage;
		aveW /= Memory;
		choiceNum = (int)(ran1(&seed) * Memory);
		choice = worker[h].list.firm[choiceNum];
		worker[h].choice = choice;
		worker[h].effort = maxF(0.0, env.eff + worker[h].abl
			+ (firm[choice].wage - aveW) / aveW
			+ firm[choice].csr * worker[h].csr);
	}
}

void productF(){
	int i,w,num;
	double sum,wage;

	for(i=0;i<NFirm;i++){
		num = 0;
		sum = 0.0;
		for(w=0;w<NWorker;w++) if(worker[w].choice == i){
			num++;
			sum += worker[w].effort;
		}
		firm[i].quantity
			 = (1.0 - env.upcsr * firm[i].csr)
			 * maxF(0.0,sum + env.beta * firm[i].stock);
		if(num != 0) wage = firm[i].quantity / num;
		else wage = 0.0;
		firm[i].wage = wage;
		for(w=0;w<NWorker;w++) if(worker[w].choice == i)
			worker[w].wage = wage;
	}
}

void consumpF(){
	int h,i,choice,choiceNum;
	double sum;

	sum = 0.0;
	for(i=0;i<NFirm;i++) sum += firm[i].quantity;
	sum /= NConsumer;

	for(h=0;h<NConsumer;h++){
		choiceNum = (int)(ran1(&seed) * Memory);
		choice = consumer[h].list.firm[choiceNum];
		consumer[h].choice = choice;
		(firm[choice].demand) += sum;
	}
}

void updateList(){
	double ave,v[Memory],minV,maxV;
	int i,j,k,flag,target,f[NFirm],o[Memory];

	minV = -1.0;
	maxV = 1.0;
// worker
	for(i=0;i<NWorker;i++){
		ave = 0.0;
		for(j=0;j<Memory;j++){
			target = worker[i].list.firm[j];
			f[j] = target;
			ave += firm[target].wage;
		}
		ave /= Memory;
		for(j=0;j<Memory;j++){
			target = f[j];
			v[j] = noiseF(Noise,(firm[target].wage - ave) / ave
			+ worker[i].csr * firm[target].csr,minV,maxV);
		}
		orderF(Memory,v,o);
		for(j=HalfMemory;j<Memory;j++){
			do{
				flag = 0;
				target = (int)(ran1(&seed) * NFirm);
				for(k=0;k<j;k++) if(f[o[k]] == target) flag = 1;
			} while(flag==1);
			f[o[j]] = target;
		}
		for(j=0;j<Memory;j++) worker[i].list.firm[j] = f[j];
	}
// consumer
	for(i=0;i<NConsumer;i++){
		ave = 0.0;
		for(j=0;j<Memory;j++){
			target = consumer[i].list.firm[j];
			f[j] = target;
			ave += firm[target].quantity;
		}
		ave /= Memory;
		for(j=0;j<Memory;j++){
			target = f[j];
			v[j] = noiseF(Noise,(firm[target].quantity - ave) / ave
			+ consumer[i].csr * firm[target].csr,minV,maxV);
		}
		orderF(Memory,v,o);
		for(j=HalfMemory;j<Memory;j++){
			do{
				flag = 0;
				target = (int)(ran1(&seed) * NFirm);
				for(k=0;k<j;k++) if(f[o[k]] == target) flag = 1;
			} while(flag==1);
			f[o[j]] = target;
		}
		for(j=0;j<Memory;j++) consumer[i].list.firm[j] = f[j];
	}
// holder
	for(i=0;i<NHolder;i++){
		ave = 0.0;
		for(j=0;j<Memory;j++){
			target = holder[i].list.firm[j];
			f[j] = target;
			ave += firm[target].demand;
		}
		ave /= Memory;
		for(j=0;j<Memory;j++){
			target = f[j];
			v[j] = noiseF(Noise,(firm[target].demand - ave) / ave
			+ holder[i].csr * firm[target].csr,minV,maxV);
		}
		orderF(Memory,v,o);
		for(j=HalfMemory;j<Memory;j++){
			do{
				flag = 0;
				target = (int)(ran1(&seed) * NFirm);
				for(k=0;k<j;k++) if(f[o[k]] == target) flag = 1;
			} while(flag==1);
			f[o[j]] = target;
		}
		for(j=0;j<Memory;j++) holder[i].list.firm[j] = f[j];
	}
}

void cutCsrF(){
	int i;
	for(i=0;i<NWorker;i++) 
//		worker[i].csr *= env.scsr;
		worker[i].csr = 0.0;
	for(i=0;i<NConsumer;i++)
//		consumer[i].csr *= env.scsr;
//		consumer[i].csr = 0.0;
//	for(i=0;i<NHolder;i++) holder[i].csr *= env.scsr;
	for(i=0;i<NHolder;i++) holder[i].csr = 0.0;
}
